### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.schema.interfaces import IVocabularyFactory

# import local interfaces

# import Zope3 packages
from zope.i18n import translate
from zope.interface import classProvides
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

# import local packages

from ztfy.i18n import _


_base_languages = (
                    (u'aa', _('Afar')),
                    (u'ab', _('Abkhazian')),
                    (u'ae', _('Avestan')),
                    (u'af', _('Afrikaans')),
                    (u'ak', _('Akan')),
                    (u'am', _('Amharic')),
                    (u'an', _('Aragonese')),
                    (u'ar', _('Arabic')),
                    (u'as', _('Assamese')),
                    (u'av', _('Avaric')),
                    (u'ay', _('Aymara')),
                    (u'az', _('Azerbaijani')),
                    (u'ba', _('Bashkir')),
                    (u'be', _('Belarusian')),
                    (u'bg', _('Bulgarian')),
                    (u'bh', _('Bihari')),
                    (u'bi', _('Bislama')),
                    (u'bm', _('Bambara')),
                    (u'bn', _('Bengali')),
                    (u'bo', _('Tibetan')),
                    (u'br', _('Breton')),
                    (u'bs', _('Bosnian')),
                    (u'ca', _('Catalan')),
                    (u'ce', _('Chechen')),
                    (u'ch', _('Chamorro')),
                    (u'co', _('Corsican')),
                    (u'cr', _('Cree')),
                    (u'cs', _('Czech')),
                    (u'cu', _('Old Church Slavonic')),
                    (u'cv', _('Chuvash')),
                    (u'cy', _('Welsh')),
                    (u'da', _('Danish')),
                    (u'de', _('German')),
                    (u'dv', _('Divehi')),
                    (u'dz', _('Dzongkha')),
                    (u'ee', _('Ewe')),
                    (u'el', _('Greek')),
                    (u'en', _('English')),
                    (u'eo', _('Esperanto')),
                    (u'es', _('Spanish')),
                    (u'et', _('Estonian')),
                    (u'eu', _('Basque')),
                    (u'fa', _('Persian')),
                    (u'ff', _('Fulah')),
                    (u'fi', _('Finnish')),
                    (u'fj', _('Fijian')),
                    (u'fo', _('Faroese')),
                    (u'fr', _('French')),
                    (u'fy', _('Western Frisian')),
                    (u'ga', _('Irish')),
                    (u'gd', _('Scottish Gaelic')),
                    (u'gl', _('Galician')),
                    (u'gn', _('Guarani')),
                    (u'gu', _('Gujarati')),
                    (u'gv', _('Manx')),
                    (u'ha', _('Hausa')),
                    (u'he', _('Hebrew')),
                    (u'hi', _('Hindi')),
                    (u'ho', _('Hiri Motu')),
                    (u'hr', _('Croatian')),
                    (u'ht', _('Haitian')),
                    (u'hu', _('Hungarian')),
                    (u'hy', _('Armenian')),
                    (u'hz', _('Herero')),
                    (u'ia', _('Interlingua')),
                    (u'id', _('Indonesian')),
                    (u'ie', _('Interlingue')),
                    (u'ig', _('Igbo')),
                    (u'ii', _('Sichuan Yi')),
                    (u'ik', _('Inupiaq')),
                    (u'io', _('Ido')),
                    (u'is', _('Icelandic')),
                    (u'it', _('Italian')),
                    (u'iu', _('Inuktitut')),
                    (u'ja', _('Japanese')),
                    (u'jv', _('Javanese')),
                    (u'ka', _('Georgian')),
                    (u'kg', _('Kongo')),
                    (u'ki', _('Kikuyu')),
                    (u'kj', _('Kwanyama')),
                    (u'kk', _('Kazakh')),
                    (u'kl', _('Kalaallisut')),
                    (u'km', _('Khmer')),
                    (u'kn', _('Kannada')),
                    (u'ko', _('Korean')),
                    (u'kr', _('Kanuri')),
                    (u'ks', _('Kashmiri')),
                    (u'ku', _('Kurdish')),
                    (u'kv', _('Komi')),
                    (u'kw', _('Cornish')),
                    (u'ky', _('Kirghiz')),
                    (u'la', _('Latin')),
                    (u'lb', _('Luxembourgish')),
                    (u'lg', _('Ganda')),
                    (u'li', _('Limburgish')),
                    (u'ln', _('Lingala')),
                    (u'lo', _('Lao')),
                    (u'lt', _('Lithuanian')),
                    (u'lu', _('Luba-Katanga')),
                    (u'lv', _('Latvian')),
                    (u'mg', _('Malagasy')),
                    (u'mh', _('Marshallese')),
                    (u'mi', _('Maori')),
                    (u'mk', _('Macedonian')),
                    (u'ml', _('Malayalam')),
                    (u'mn', _('Mongolian')),
                    (u'mo', _('Moldavian')),
                    (u'mr', _('Marathi')),
                    (u'ms', _('Malay')),
                    (u'mt', _('Maltese')),
                    (u'my', _('Burmese')),
                    (u'na', _('Nauru')),
                    (u'nb', _('Norwegian Bokmal')),
                    (u'nd', _('North Ndebele')),
                    (u'ne', _('Nepali')),
                    (u'ng', _('Ndonga')),
                    (u'nl', _('Dutch')),
                    (u'nn', _('Norwegian Nynorsk')),
                    (u'no', _('Norwegian')),
                    (u'nr', _('South Ndebele')),
                    (u'nv', _('Navajo')),
                    (u'ny', _('Chichewa')),
                    (u'oc', _('Occitan')),
                    (u'oj', _('Ojibwa')),
                    (u'om', _('Oromo')),
                    (u'or', _('Oriya')),
                    (u'os', _('Ossetian')),
                    (u'pa', _('Panjabi')),
                    (u'pi', _('Pali')),
                    (u'pl', _('Polish')),
                    (u'ps', _('Pashto')),
                    (u'pt', _('Portuguese')),
                    (u'qu', _('Quechua')),
                    (u'rm', _('Raeto-Romance')),
                    (u'rn', _('Kirundi')),
                    (u'ro', _('Romanian')),
                    (u'ru', _('Russian')),
                    (u'rw', _('Kinyarwanda')),
                    (u'sa', _('Sanskrit')),
                    (u'sc', _('Sardinian')),
                    (u'sd', _('Sindhi')),
                    (u'se', _('Northern Sami')),
                    (u'sg', _('Sango')),
                    (u'si', _('Sinhalese')),
                    (u'sk', _('Slovak')),
                    (u'sl', _('Slovene')),
                    (u'sm', _('Samoan')),
                    (u'sn', _('Shona')),
                    (u'so', _('Somali')),
                    (u'sq', _('Albanian')),
                    (u'sr', _('Serbian')),
                    (u'ss', _('Swati')),
                    (u'st', _('Sotho')),
                    (u'su', _('Sundanese')),
                    (u'sv', _('Swedish')),
                    (u'sw', _('Swahili')),
                    (u'ta', _('Tamil')),
                    (u'te', _('Telugu')),
                    (u'tg', _('Tajik')),
                    (u'th', _('Thai')),
                    (u'ti', _('Tigrinya')),
                    (u'tk', _('Turkmen')),
                    (u'tl', _('Tagalog')),
                    (u'tn', _('Tswana')),
                    (u'to', _('Tonga')),
                    (u'tr', _('Turkish')),
                    (u'ts', _('Tsonga')),
                    (u'tt', _('Tatar')),
                    (u'tw', _('Twi')),
                    (u'ty', _('Tahitian')),
                    (u'ug', _('Uighur')),
                    (u'uk', _('Ukrainian')),
                    (u'ur', _('Urdu')),
                    (u'uz', _('Uzbek')),
                    (u've', _('Venda')),
                    (u'vi', _('Vietnamese')),
                    (u'vo', _('Volapuk')),
                    (u'wa', _('Walloon')),
                    (u'wo', _('Wolof')),
                    (u'xh', _('Xhosa')),
                    (u'yi', _('Yiddish')),
                    (u'yo', _('Yoruba')),
                    (u'za', _('Zhuang')),
                    (u'zh', _('Chinese')),
                    (u'zu', _('Zulu'))
                   )



class BaseLanguagesVocabulary(SimpleVocabulary):
    """Base languages vocabulary, matching ISO 639-1 languages codes"""
    classProvides(IVocabularyFactory)

    def __init__(self, *args, **kw):
        terms = sorted([SimpleTerm(v, t, translate(t)) for v, t in _base_languages])
        super(BaseLanguagesVocabulary, self).__init__(sorted(terms, key=lambda x: x.title))


_languages = (
               (u'aa-ET', _("Afar (Ethiopia)")),
               (u'ab-GE', _("Abkhazian")),
               (u'ae-IR', _("Avestan (Iran)")),
               (u'af-ZA', _('Afrikaans')),
               (u'ak-GH', _("Akan (Ghana)")),
               (u'am-ET', _('Amharic (Ethiopia)')),
               (u'an-ES', _("Aragonese (Spain)")),
               (u'ar-AE', _('Arabic (United Arab Emirates)')),
               (u'ar-BH', _('Arabic (Bahrain)')),
               (u'ar-DZ', _('Arabic (Algeria)')),
               (u'ar-EG', _('Arabic (Egypt)')),
               (u'ar-IQ', _('Arabic (Iraq)')),
               (u'ar-JO', _('Arabic (Jordan)')),
               (u'ar-KW', _('Arabic (Kuwait)')),
               (u'ar-LB', _('Arabic (Lebanon)')),
               (u'ar-LY', _('Arabic (Libyan Arab Jamahiriya)')),
               (u'ar-MA', _('Arabic (Morocco)')),
               (u'ar-OM', _('Arabic (Oman)')),
               (u'ar-QA', _('Arabic (Qatar)')),
               (u'ar-SA', _('Arabic (Saudi Arabia)')),
               (u'ar-SY', _('Arabic (Syrian Arab Republic)')),
               (u'ar-TN', _('Arabic (Tunisia)')),
               (u'ar-YE', _('Arabic (Yemen)')),
               (u'arn-CL', _('Mapudungun (Chile)')),
               (u'as-IN', _('Assamese (India)')),
               (u'ast-ES', _('Asturien')),
               (u'av-RU', _("Avaric (Russia)")),
               (u'ay-BO', _("Aymara (Bolivia)")),
               (u'az-AZ-Cyrl', _('Azerbaijani (Cyrillic)')),
               (u'az-AZ-Latn', _('Azerbaijani (Latin)')),
               (u'ba-RU', _('Bashkir (Russia)')),
               (u'be-BY', _('Belarusian (Belarus)')),
               (u'ber-DZ', _('Berber (Algeria, Latin)')),
               (u'bg-BG', _('Bulgarian')),
               (u'bh-IN', _("Bihari (India)")),
               (u'bi-VU', _("Bislama (Vanuatu)")),
               (u'bm-ML', _("Bambara (Mali)")),
               (u'bn-IN', _('Bengali (India)')),
               (u'bo-BT', _('Tibetan (Bhutan)')),
               (u'bo-CN', _('Tibetan (China)')),
               (u'br-FR', _('Breton')),
               (u'bs-BA-Cyrl', _('Bosnian (Bosnia Herzegovina, Cyrillic)')),
               (u'bs-BA-Latn', _('Bosnian (Bosnie Herzegovina, Latin)')),
               (u'ca-AD', _('Catalan (Andorra)')),
               (u'ca-ES', _('Catalan (Spain)')),
               (u'ca-FR', _('Catalan (France)')),
               (u'ce-RU', _('Chechen')),
               (u'ch-US', _('Chamorro')),
               (u'co-FR', _('Corsican (France)')),
               (u'cr-CA', _('Cree (Canada)')),
               (u'cs-CZ', _('Czech')),
               (u'cu-BG', _('Old Slavonic')),
               (u'cv-RU', _('Chuvash (Russia)')),
               (u'cy-GB', _('Welsh')),
               (u'da-DK', _('Danish')),
               (u'de-AT', _('German (Austria)')),
               (u'de-CH', _('German (Swotzerland)')),
               (u'de-DE', _('German (Germany)')),
               (u'de-LI', _('German (Liechtenstein)')),
               (u'de-LU', _('German (Luxembourg)')),
               (u'dv-MV', _('Divehi (Maldives)')),
               (u'dz-BT', _('Dzongkha (Bhutan)')),
               (u'ee-GH', _('Ewe (Ghana)')),
               (u'el-GR', _('Greek')),
               (u'en-AU', _('English (Australia)')),
               (u'en-BZ', _('English (Belize)')),
               (u'en-CA', _('English (Canada)')),
               (u'en-CB', _('English (Caraibes)')),
               (u'en-GB', _('English (United Kingdom)')),
               (u'en-IE', _('English (Ireland)')),
               (u'en-IN', _('English (India)')),
               (u'en-JA', _('English (Jamaica)')),
               (u'en-MY', _('English (Malaysia)')),
               (u'en-NZ', _('English (New Zealand)')),
               (u'en-PH', _('English (Philippines)')),
               (u'en-SG', _('English (Singapore)')),
               (u'en-TT', _('English (Trinidad)')),
               (u'en-US', _('English (United States)')),
               (u'en-ZA', _('English (South Africa)')),
               (u'en-ZW', _('English (Zimbabwe)')),
               (u'eo', _('Esperanto')),
               (u'es-AR', _('Spanish (Argentina)')),
               (u'es-BO', _('Spanish (Bolivia)')),
               (u'es-CL', _('Spanish (Chile)')),
               (u'es-CO', _('Spanish (Colombia)')),
               (u'es-CR', _('Spanish (Costa Rica)')),
               (u'es-DO', _('Spanish (Dominican Republic)')),
               (u'es-EC', _('Spanish (Ecuador)')),
               (u'es-ES', _('Spanish (Spain)')),
               (u'es-ES-ts', _('Spanish (Spain, Traditional)')),
               (u'es-GT', _('Spanish (Guatemala)')),
               (u'es-HN', _('Spanish (Honduras)')),
               (u'es-MX', _('Spanish (Mexico)')),
               (u'es-NI', _('Spanish (Nicaragua)')),
               (u'es-PA', _('Spanish (Panama)')),
               (u'es-PE', _('Spanish (Peru)')),
               (u'es-PR', _('Spanish (Puerto Rico)')),
               (u'es-PY', _('Spanish (Paraguay)')),
               (u'es-SV', _('Spanish (El Salvador)')),
               (u'es-UR', _('Spanish (Uruguay)')),
               (u'es-US', _('Spanish (United States)')),
               (u'es-VE', _('Spanish (Venezuela)')),
               (u'et-EE', _('Estonian (Estonia)')),
               (u'eu-ES', _('Basque')),
               (u'fa-IR', _('Persian (Iran)')),
               (u'fi-FI', _('Finish')),
               (u'fil-PH', _('Philippine')),
               (u'fj-FJ', _('Fijian')),
               (u'fo-FO', _('Faroese (Faroe Islands)')),
               (u'fr-FR', _('French (France)')),
               (u'fr-BE', _('French (Belgium)')),
               (u'fr-CA', _('French (Canada)')),
               (u'fr-CH', _('French (Switzerland)')),
               (u'fr-LU', _('French (Luxembourg)')),
               (u'fr-MC', _('French (Monaco)')),
               (u'fur-IT', _('Frioulan (Italia)')),
               (u'fy-NL', _('Frisian (Netherlands)')),
               (u'ga-IE', _('Irish')),
               (u'gbz-AF', _('Dari (Afghanistan)')),
               (u'gd-GB', _('Gaelic (Scotland)')),
               (u'gl-ES', _('Galician')),
               (u'gsw-FR', _('Alsatian (France)')),
               (u'gu-IN', _('Gujarati (India)')),
               (u'ha-NG-Latn', _('Hausa (Nigeria)')),
               (u'he-IL', _('Hebrew (Israel)')),
               (u'hi-IN', _('Hindi (Inde)')),
               (u'hr-BA', _('Croatian (Bosnia Herzegovina, Latin)')),
               (u'hr-HR', _('Croatian (Croatia)')),
               (u'hu-HU', _('Hungarian')),
               (u'hy-AM', _('Armenian (Armenia)')),
               (u'id-ID', _('Indonesian')),
               (u'ii-CN', _('Yi (China)')),
               (u'iu-CA-Cans', _('Inuktitut (Canada, Syllabic)')),
               (u'iu-CA-Latn', _('Inuktitut (Canada, Latin)')),
               (u'is-IS', _('Icelandic')),
               (u'it-CH', _('Italian (Switzerland)')),
               (u'it-IT', _('Italian (Italia)')),
               (u'ja-JP', _('Japanese')),
               (u'ja-JP-mac', _('Japanese (Japan, Mac)')),
               (u'ka-GE', _('Georgian')),
               (u'kk-KZ', _('Kazakh (Kazakhstan)')),
               (u'kl-GL', _('Greenlandic')),
               (u'km-KH', _('Khmer (Cambodia)')),
               (u'kn-IN', _('Kannada (India)')),
               (u'ko-KR', _('Korean (Korea)')),
               (u'kok-IN', _('Konkani (India)')),
               (u'kw-GB', _('Cornish')),
               (u'ky-KG', _('Kirghiz (Kyrgizstan)')),
               (u'lb-LU', _('Luxembourgish')),
               (u'lo-LA', _('Lao')),
               (u'lt-LT', _('Lituanian')),
               (u'lv-LV', _('Latvian')),
               (u'mi-NZ', _('Maori (New Zealand)')),
               (u'mk-MK', _('Macedonian')),
               (u'ml-IN', _('Malayalam (Inde)')),
               (u'mn-CN', _('Mongolian (China)')),
               (u'mn-MN', _('Mongolian (Mongolia)')),
               (u'moh-CA', _('Mohawk (Canada)')),
               (u'mr-IN', _('Marathi (Inde)')),
               (u'ms-BN', _('Malay (Brunei)')),
               (u'ms-MY', _('Malay')),
               (u'mt-MT', _('Maltese')),
               (u'my-MM', _('Burmese (Myanmar)')),
               (u'nb-NO', _('Norwegian (Bokmal)')),
               (u'nn-NO', _('Norwegian (Nynorsk)')),
               (u'ne-NP', _('Nepali')),
               (u'nl-BE', _('Dutch (Belgium)')),
               (u'nl-NL', _('Dutch (Netherlands)')),
               (u'oc-FR', _('Occitan (France)')),
               (u'or-IN', _('Oriya (India)')),
               (u'pa-IN', _('Punjabi (India)')),
               (u'pl-PL', _('Polish')),
               (u'ps-AF', _('Pachto (Afghanistan)')),
               (u'pt-BR', _('Portuguese (Brazil)')),
               (u'pt-PT', _('Portuguese (Portugal)')),
               (u'que-BO', _('Quechua (Bolivia)')),
               (u'que-EC', _('Quechua (Ecuador)')),
               (u'que-PE', _('Quechua (Peru)')),
               (u'rm-CH', _('Romansh')),
               (u'ro-RO', _('Romanian')),
               (u'ru-RU', _('Russian')),
               (u'rw-RW', _('Kinyarwanda (Rwanda)')),
               (u'sa-IN', _('Sanskrit (Inde)')),
               (u'sah-RU', _('Yakut (Russia)')),
               (u'se-FI', _('Sami (Northern, Finland)')),
               (u'se-NO', _('Sami (Northern, Norway)')),
               (u'se-SE', _('Sami (Northern, Sweden)')),
               (u'si-LK', _('Sinhala (Sri Lanka)')),
               (u'sk-SK', _('Slovak')),
               (u'sl-SI', _('Slovenian')),
               (u'sr-BA-Cyrl', _('Serbian (Bosnia Herzegovina, Cyrillic)')),
               (u'sr-BA-Latn', _('Serbian (Bosnia Herzegovina, Latin)')),
               (u'sr-SP-Cyrl', _('Serbian (Serbia and Montenegro, Cyrillic)')),
               (u'sr-SP-Latn', _('Serbian (Serbia and Montenegro, Latin)')),
               (u'sma-NO', _('Sami (Southern, Norway)')),
               (u'sma-SE', _('Sami (Southern, Sweden)')),
               (u'smj-NO', _('Sami (Lule, Norway)')),
               (u'smj-SE', _('Sami (Lule, Sweden)')),
               (u'smn-FI', _('Sami (Inari, Finland)')),
               (u'sms-FI', _('Sami (Skolt, Finland)')),
               (u'sq-AL', _('Albanian')),
               (u'sv-FI', _('Swedish (Finland)')),
               (u'sv-SE', _('Swedish (Sweden)')),
               (u'sw-KE', _('Swahili (Kenya)')),
               (u'syr-SY', _('Syriac')),
               (u'ta-IN', _('Tamil (India)')),
               (u'te-IN', _('Telugu (India)')),
               (u'tg-TJ-Cyrl', _('Tajik (Tajikistan)')),
               (u'th-TH', _('Thai (Thailand)')),
               (u'tk-TM', _('Turkmen (Turkmenistan)')),
               (u'tn-ZA', _('Tswana (South Africa)')),
               (u'tr-TR', _('Turkish')),
               (u'tt-RU', _('Tatar (Russia)')),
               (u'ug-CN', _('Uighur (China)')),
               (u'uk-UA', _('Ukrainian')),
               (u'ur-IN', _('Urdu (Inde)')),
               (u'ur-PK', _('Urdu (Pakistan)')),
               (u'uz-UZ-Cyrl', _('Uzbek (Uzbekistan, Cyrillic)')),
               (u'uz-UZ-Latn', _('Uzbek (Uzbekistan, Latin)')),
               (u'vi-VN', _('Vietnamese')),
               (u'wen-DE', _('Sorbian (Germany)')),
               (u'wo-SN', _('Wolof (Senegal)')),
               (u'xh-ZA', _('Xhosa (South Africa)')),
               (u'yo-NG', _('Yoruba (Nigeria)')),
               (u'zh-CHS', _('Chinese (Simplified)')),
               (u'zh-CHT', _('Chinese (Traditional)')),
               (u'zh-CN', _('Chinese (China)')),
               (u'zh-HK', _('Chinese (Hong Kong, China)')),
               (u'zh-MO', _('Chinese (Macao)')),
               (u'zh-SG', _('Chinese (Singapore)')),
               (u'zh-TW', _('Chinese (Taiwan)')),
               (u'zu-ZA', _('Zulu (South Africa)'))
             )


class LanguagesVocabulary(SimpleVocabulary):

    classProvides(IVocabularyFactory)

    def __init__(self, *args, **kw):
        terms = sorted([SimpleTerm(v, t, translate(t)) for v, t in _languages])
        super(LanguagesVocabulary, self).__init__(sorted(terms, key=lambda x: x.title))
